#include <cstdio>
#include <string>
#include <utility>
#include <vector>

// note: this is just an example of using RAII
// it is much better to use <iostream> for I/O purposes in C++

class c_file {
	std::FILE* ptr;

public:
	c_file(const char* filename, const char* mode)
	  : ptr(std::fopen(filename, mode)) {}

	bool is_open() const { return ptr != nullptr; }

	// no copying ...
	c_file(const c_file&) = delete;
	c_file& operator=(const c_file&) = delete;

	// ... but moving makes sense here
	c_file(c_file&& other) noexcept
	  : ptr(std::exchange(other.ptr, nullptr)) {}

	c_file& operator=(c_file&& other) noexcept {
		close();
		ptr = std::exchange(other.ptr, nullptr);
		return *this;
	}

	~c_file() { close(); }

	void close() {
		if (ptr != nullptr) {
			std::fclose(ptr);
			ptr = nullptr;
		}
	}

	size_t read(void* buffer, size_t size, size_t count) {
		return fread(buffer, size, count, ptr);
	}

	size_t write(const void* buffer, size_t size, size_t count) {
		return fwrite(buffer, size, count, ptr);
	}
};

int main() {
	std::vector<c_file> outputs;

	// vector reallocation is OK, we have a move constructor!
	outputs.emplace_back("one.txt", "w");
	outputs.emplace_back("two.txt", "w");
	outputs.emplace_back("three.txt", "w");

	for (auto& file : outputs) {
		if (file.is_open())
			file.write("Hello!\n", sizeof(char), 7);
	}
}
